# CodeAbility Sharing Platform V0.0

This is an initial version V0.0 of the CodeAbility Sharing platform. 
The goal of this initial version is to 

- allow an initial exchange of resources (e.g., lab exercises, lecture slides, collections of links, or Artemis exercises),
- give us a first concrete impression on available material to be shared.

This prototype uses the community edition of GitLab, which, among others, provides a web-based Git repository manager. 
Therefore, the well-known and established processes of Git can be used for sharing resources. In the following, the registration, usage, and plans are discussed.

Our plans are to extend the usability and functionality of the sharing platform step by step to make it more suitable to the project's needs.

As a very first beginning the platform 

- is restricted to the closed user group of project partners,
- is expected to contain only material under the [CC-BY-SA V4.0](https://creativecommons.org/licenses/by-sa/4.0/) license. (If this condition is a severe problem for you, please let us know.)

We will lift these restrictions step-by-step and add further functionality when we progress with the platform.

## 1. Registration

The sharing platform is available on https://sharing-codeability.uibk.ac.at/. We would kindly ask you to use the self-registration function and inform us via email (sharing-codeability@uibk.ac.at). As soon as we verified your access permission, you can access all resources, which your colleges have been sharing. 

## 2. Usage

After your successful registration, a lot of features are available for you. In this evaluation phase, we focus on the three main use-cases - publish, view, and discuss resources. Those are described in the following.

### 2.1 Publish Resources

For publishing resources, you can create new repositories in the group `sharing`/`<your university>`. Please, use an informative repository name when creating 
a new repository (currently, you cannot search for specific terms across all repositories). 
A repository can contain various resources like a course, an exercise, or a list of links. 
However, please ensure that you only join resources in a single repository that are somehow related and do not use a single repository as a large data container. 

Moreover, you should include a `Readme.md`-File in each repository that comprises a description of the repository's content and possibly the requirements (e.g., to solve the exercises). The latest version should always be available in the master branch. For further iterations of the course, version tags can be used.
We recommend additional files

   - `LICENSE` (may contain the legal license terms)
   - `metadata.yaml` (as a first example see [some artemis exercise](https://sharing-codeability.uibk.ac.at/sharing/codeability-sharing-platform/-/wikis/technical/MetaData%20Documentation)). The format ist described [here](https://sharing-codeability.uibk.ac.at/sharing/codeability-sharing-platform/-/wikis/technical/MetaData%20Documentation)

To improve the search experience of relevant repositories, you can provide topics and a project description (see `Settings`/`General`). As a reference, you can take a look at a [demo repository](https://sharing-codeability.uibk.ac.at/sharing/university-innsbruck/java/general/junit-quality-tests-exercise).
Note that in this version, all resources are published under the license CC BY-SA.

### 2.2 View Resources

After an interesting project was found in the shared group `sharing`, the content can be viewed in the web browser, or the repository can be downloaded by Git means into your local repository.

### 2.3 Discuss Resources

For discussing the material or asking questions about the resources of a repository, issues can be used.


## 3. Plans

For the immediate next steps, we think that the following extensions would be beneficial.

* Full-text search across all repositories
* Support for EduID
* Collection and search for metadata

These features will be made available as soon as they are available.


# The gitsearch Application

This application was generated using JHipster 6.10.0, you can find documentation and help at [https://www.jhipster.tech/documentation-archive/v6.10.0](https://www.jhipster.tech/documentation-archive/v6.10.0).

## Development

Before you can build this project, you must install and configure the following dependencies on your machine:

1. [Node.js][]: We use Node to run a development web server and build the project.
   Depending on your system, you can install Node either from source or as a pre-packaged bundle.

After installing Node, you should be able to run the following command to install development tools.
You will only need to run this command when dependencies change in [package.json](package.json).

```
npm install
```

We use npm scripts and [Webpack][] as our build system.

Run the following commands in two separate terminals to create a blissful development experience where your browser
auto-refreshes when files change on your hard drive.

```

./mvnw


npm start
```

Npm is also used to manage CSS and JavaScript dependencies used in this application. You can upgrade dependencies by
specifying a newer version in [package.json](package.json). You can also run `npm update` and `npm install` to manage dependencies.
Add the `help` flag on any command to see how you can use it. For example, `npm help update`.

The `npm run` command will list all of the scripts available to run for this project.

### PWA Support

JHipster ships with PWA (Progressive Web App) support, and it's turned off by default. One of the main components of a PWA is a service worker.

The service worker initialization code is commented out by default. To enable it, uncomment the following code in `src/main/webapp/index.html`:

```html
<script>
  if ('serviceWorker' in navigator) {
    navigator.serviceWorker.register('./service-worker.js').then(function () {
      console.log('Service Worker Registered');
    });
  }
</script>
```

Note: [Workbox](https://developers.google.com/web/tools/workbox/) powers JHipster's service worker. It dynamically generates the `service-worker.js` file.

### Managing dependencies

For example, to add [Leaflet][] library as a runtime dependency of your application, you would run following command:

```
npm install --save --save-exact leaflet
```

To benefit from TypeScript type definitions from [DefinitelyTyped][] repository in development, you would run following command:

```
npm install --save-dev --save-exact @types/leaflet
```

Then you would import the JS and CSS files specified in library's installation instructions so that [Webpack][] knows about them:
Edit [src/main/webapp/app/vendor.ts](src/main/webapp/app/vendor.ts) file:

```
import 'leaflet/dist/leaflet.js';
```

Edit [src/main/webapp/content/scss/vendor.scss](src/main/webapp/content/scss/vendor.scss) file:

```
@import '~leaflet/dist/leaflet.css';
```

Note: There are still a few other things remaining to do for Leaflet that we won't detail here.

For further instructions on how to develop with JHipster, have a look at [Using JHipster in development][].

### Using Angular CLI

You can also use [Angular CLI][] to generate some custom client code.

For example, the following command:

```
ng generate component my-component
```

will generate few files:

```
create src/main/webapp/app/my-component/my-component.component.html
create src/main/webapp/app/my-component/my-component.component.ts
update src/main/webapp/app/app.module.ts
```

## Building for production

### Packaging as jar

To build the final jar and optimize the gitsearch application for production, run:

```

./mvnw -Pprod clean verify


```

This will concatenate and minify the client CSS and JavaScript files. It will also modify `index.html` so it references these new files.
To ensure everything worked, run:

```

java -jar target/*.jar


```

Then navigate to [http://localhost:8080](http://localhost:8080) in your browser.

Refer to [Using JHipster in production][] for more details.

### Packaging as war

To package your application as a war in order to deploy it to an application server, run:

```

./mvnw -Pprod,war clean verify


```

## Testing

To launch your application's tests, run:

```
./mvnw verify
```

### Client tests

Unit tests are run by [Jest][] and written with [Jasmine][]. They're located in [src/test/javascript/](src/test/javascript/) and can be run with:

```
npm test
```

For more information, refer to the [Running tests page][].

### Code quality

Sonar is used to analyse code quality. You can start a local Sonar server (accessible on http://localhost:9001) with:

```
docker-compose -f src/main/docker/sonar.yml up -d
```

You can run a Sonar analysis with using the [sonar-scanner](https://docs.sonarqube.org/display/SCAN/Analyzing+with+SonarQube+Scanner) or by using the maven plugin.

Then, run a Sonar analysis:

```
./mvnw -Pprod clean verify sonar:sonar
```

If you need to re-run the Sonar phase, please be sure to specify at least the `initialize` phase since Sonar properties are loaded from the sonar-project.properties file.

```
./mvnw initialize sonar:sonar
```

For more information, refer to the [Code quality page][].

## Using Docker to simplify development (optional)

You can use Docker to improve your JHipster development experience. A number of docker-compose configuration are available in the [src/main/docker](src/main/docker) folder to launch required third party services.

For example, to start a mysql database in a docker container, run:

```
docker-compose -f src/main/docker/mysql.yml up -d
```

To stop it and remove the container, run:

```
docker-compose -f src/main/docker/mysql.yml down
```

You can also fully dockerize your application and all the services that it depends on.
To achieve this, first build a docker image of your app by running:

```
./mvnw -Pprod verify jib:dockerBuild
```

Then run:

```
docker-compose -f src/main/docker/app.yml up -d
```

For more information refer to [Using Docker and Docker-Compose][], this page also contains information on the docker-compose sub-generator (`jhipster docker-compose`), which is able to generate docker configurations for one or several JHipster applications.

## Continuous Integration (optional)

To configure CI for your project, run the ci-cd sub-generator (`jhipster ci-cd`), this will let you generate configuration files for a number of Continuous Integration systems. Consult the [Setting up Continuous Integration][] page for more information.

[jhipster homepage and latest documentation]: https://www.jhipster.tech
[jhipster 6.10.0 archive]: https://www.jhipster.tech/documentation-archive/v6.10.0
[using jhipster in development]: https://www.jhipster.tech/documentation-archive/v6.10.0/development/
[using docker and docker-compose]: https://www.jhipster.tech/documentation-archive/v6.10.0/docker-compose
[using jhipster in production]: https://www.jhipster.tech/documentation-archive/v6.10.0/production/
[running tests page]: https://www.jhipster.tech/documentation-archive/v6.10.0/running-tests/
[code quality page]: https://www.jhipster.tech/documentation-archive/v6.10.0/code-quality/
[setting up continuous integration]: https://www.jhipster.tech/documentation-archive/v6.10.0/setting-up-ci/
[node.js]: https://nodejs.org/
[yarn]: https://yarnpkg.org/
[webpack]: https://webpack.github.io/
[angular cli]: https://cli.angular.io/
[browsersync]: https://www.browsersync.io/
[jest]: https://facebook.github.io/jest/
[jasmine]: https://jasmine.github.io/2.0/introduction.html
[protractor]: https://angular.github.io/protractor/
[leaflet]: https://leafletjs.com/
[definitelytyped]: https://definitelytyped.org/

